/**
 * Javascript to make Customizer preview reflect changes without refresh.
 * @package Twenty8teen
 */

( function( $ ) {
	// Header text color.
	wp.customize( 'header_textcolor', function( setting ) {
		var hStyle = document.createElement( 'style' );
		document.head.appendChild( hStyle );
		setting.bind( function( newval ) {
			if ( 'blank' === newval ) {
				$( '.site-title, .site-description' ).css( {
					'clip': 'rect(1px, 1px, 1px, 1px)',
					'position': 'absolute'
				} );
			} else {
				$( '.site-title, .site-description' ).css( {
					'clip': 'auto',
					'position': 'relative'
				} );
				hStyle.innerHTML = ':root { --header_textcolor: ' + newval + ' }';
			}
		} );
	} );

	wp.customize( 'background_color', function( setting ) {
		var bStyle = document.createElement( 'style' );
		document.head.appendChild( bStyle );
		setting.bind( function( newval ) {
			bStyle.innerHTML = ':root { --background_color: ' + newval + ' }';
		} );
	} );
	wp.customize( 'accent_color', function( setting ) {
		var bStyle = document.createElement( 'style' );
		document.head.appendChild( bStyle );
		setting.bind( function( newval ) {
			var out = 'radial-gradient(circle, rgba(250, 250, 250, 0) calc(100% - 200px), ' + newval + ')';
			bStyle.innerHTML = '.vignette::before {background-image: ' + out + '}' +
				':root { --accent_color: ' + newval + ' }';
			// In Edge 15 is not possible to use css variables in pseudo elements.
		} );
	} );
	wp.customize( 'body_textcolor', function( setting ) {
		var bStyle = document.createElement( 'style' );
		document.head.appendChild( bStyle );
		setting.bind( function( newval ) {
			bStyle.innerHTML = ':root { --body_textcolor: ' + newval + ' }';
		} );
	} );
	wp.customize( 'link_color', function( setting ) {
		var aStyle = document.createElement( 'style' );
		document.head.appendChild( aStyle );
		setting.bind( function( newval ) {
			aStyle.innerHTML = ':root { --link_color: ' + newval + ' }';
		} );
	} );

	wp.customize( 'identimage_alpha', function( setting ) {
		var bStyle = document.createElement( 'style' );
		document.head.appendChild( bStyle );
		setting.bind( function( newval ) {
			bStyle.innerHTML = ':root { --identimage_alpha: ' + newval + ' }';
		} );
	} );
	wp.customize( 'show_vignette', function( setting ) {
		setting.bind( function( show ) {
			$( 'body' ).toggleClass( 'vignette', show );
		} );
	} );
	wp.customize( 'show_header_imagebehind', function( setting ) {
		setting.bind( function( show ) {
			$( 'body' ).toggleClass( 'header-behind', show );
			$( '.header-image' ).toggleClass( 'image-behind', show );
		} );
	} );
	wp.customize( 'show_as_cards', function( setting ) {
		setting.bind( function( show ) {
			$( '.entry' ).toggleClass( 'cards', show );
		} );
	} );
	wp.customize( 'switch_sidebar', function( setting ) {
		setting.bind( function( show ) {
			$( 'body' ).toggleClass( 'sidebar-leading', show );
		} );
	} );

	var areaToggle = function( selector, classes ) {
		if ( classes.indexOf( ',' ) === -1 ) {
			classes = ',' + classes;
		}
		var list = classes.split( ',' );
		if ( list[0] ) {
			$( selector ).removeClass( list[0] );
		}
		if ( list[1] ) {
			$( selector ).addClass( list[1] );
		}
	};
	wp.customize( 'area_classes[header]', function( setting ) {
		setting.bind( function( classes ) {
			areaToggle( '#masthead', classes );
		} );
	} );
	wp.customize( 'area_classes[main]', function( setting ) {
		setting.bind( function( classes ) {
			areaToggle( 'main', classes );
		} );
	} );
	wp.customize( 'area_classes[content]', function( setting ) {
		setting.bind( function( classes ) {
			areaToggle( '#content', classes );
		} );
	} );
	wp.customize( 'area_classes[comments]', function( setting ) {
		setting.bind( function( classes ) {
			areaToggle( '#comments', classes );
		} );
	} );
	wp.customize( 'area_classes[sidebar]', function( setting ) {
		setting.bind( function( classes ) {
			areaToggle( '#sidebar', classes );
		} );
	} );
	wp.customize( 'area_classes[footer]', function( setting ) {
		setting.bind( function( classes ) {
			areaToggle( '#footer', classes );
		} );
	} );
	wp.customize( 'area_classes[widgets]', function( setting ) {
		setting.bind( function( classes ) {
			areaToggle( '.widget', classes );
		} );
	} );

} )( jQuery );
